@@ -6,7 +6,7 @@ DOCSIS::ConfigFile - Decodes and encodes DOCSIS config-files
 
 =head1 VERSION
 
-Version 0.55
+Version 0.5503
 
 =head1 SYNOPSIS
 
@@ -40,7 +40,7 @@ use DOCSIS::ConfigFile::Syminfo;
 use DOCSIS::ConfigFile::Decode;
 use DOCSIS::ConfigFile::Encode;
 
-our $VERSION = '0.55';
+our $VERSION = '0.5503';
 our $TRACE   = 0;
 
 =head1 METHODS
@@ -131,38 +131,42 @@ sub _decode_loop {
     my $FH           = $self->{'decode_fh'};
     my $cfg          = [];
 
-    BYTE:
+    CODE:
     while($total_length > 0) {
-        my($code, $length, $syminfo, $value, $nested, $method, $func);
+        my($syminfo, $value, $nested, $method, $func);
 
-        unless(my $bytes = read $FH, $code, 1) {
-            unless(defined $bytes) {
-                $self->logger(error => "Could not read 'code': $!");
-            }
-            last BYTE;
-        }
-        unless(my $bytes = read $FH, $length, 1) {
-            unless(defined $bytes) {
-                $self->logger(error => "Could not read 'length': $!");
-            }
-            last BYTE;
-        }
+        my $code   = $self->_read_code($FH) or last CODE;
+        my $length = $self->_read_length($FH, $code) or last CODE;
 
-        $code          = unpack("C", $code);
-        $length        = unpack("C", $length) or next BYTE;
         $total_length -= $length + 2;
         $syminfo       = Syminfo->from_code($code, $p_code);
 
         if($syminfo->func eq 'nested') {
             $nested = $self->_decode_loop($length, $syminfo->code);
         }
-        elsif($func = Decode->can($syminfo->func)) {
-            read($FH, my $data, $length);
-            ($value, $nested) = $func->($data);
-        }
         else {
-            $self->logger(debug => "Decode function does not exist");
-            $syminfo->undefined_func($code, $p_code);
+            my $bytes = read($FH, my $data, $length);
+
+            if($bytes != $length) {
+                $self->logger(
+                    fatal => "Read $bytes bytes, instead of $length"
+                );
+                next CODE;
+            }
+
+            if($func = Decode->can($syminfo->func)) {
+                ($value, $nested) = $func->($data);
+            }
+            else {
+                $data = Decode->can('string')->($data);
+                $self->logger(fatal => join "\n",
+                    "Decode function does not exist.",
+                    ">> Type: $code (pId=$p_code)",
+                    ">> Length: $length",
+                    ">> Data: $data"
+                );
+                $value = "Unknown TLV: $code, $length, $data";
+            }
         }
     
         if(defined $value or defined $nested) {
@@ -180,6 +184,47 @@ sub _decode_loop {
     return $cfg;
 }
 
+sub _read_code {
+    my $self = shift;
+    my $FH   = shift;
+    my $code;
+
+    unless(my $bytes = read $FH, $code, 1) {
+        unless(defined $bytes) {
+            $self->logger(error => "Could not read 'code': $!");
+        }
+        return;
+    }
+
+    return unpack("C", $code);
+}
+
+sub _read_length {
+    my $self  = shift;
+    my $FH    = shift;
+    my $code  = shift;
+    my $bytes = $code == 64 ? 2 : 1;
+    my $length;
+
+    unless($bytes = read $FH, $length, $bytes) {
+        unless(defined $bytes) {
+            $self->logger(error => "Could not read 'length': $!");
+        }
+        return;
+    }
+
+    # Document: PKT-SP-PROV1.5-I03-070412
+    # Chapter:  9.1 MTA Configuration File
+    if($bytes == 2) {
+        $length = unpack "n", $length or return;
+    }
+    else {
+        $length = unpack "C", $length or return;
+    }
+
+    return $length;
+}
+
 sub _value_to_cfg {
     my $self    = shift;
     my $syminfo = shift;
@@ -319,8 +364,8 @@ sub _encode_loop {
         }
 
         if($syminfo->l_limit or $syminfo->u_limit) {
-            my $value = ($tlv->{'value'} =~ /\D/) ? hex $tlv->{'value'}
-                      :                                 $tlv->{'value'};
+            my $value = ($tlv->{'value'} =~ /\D/) ? length $tlv->{'value'}
+                      :                                    $tlv->{'value'};
             if($value > $syminfo->u_limit) {
                 $self->logger(error => "Value too high: $name=$value");
                 next TLV;
@@ -336,14 +381,10 @@ sub _encode_loop {
             next TLV;
         }
 
-        $length = pack "C", int(@$data);
+        $length = (@$data > 255) ? pack("n", int @$data)
+                :                  pack("C", int @$data);
         $value  = pack "C*", @$data;
 
-        if(length $value > 255) {
-            $self->logger(error => "Value is too long in TLV#$i");
-            next TLV;
-        }
-
         $binstring .= "$type$length$value";
 
         $self->logger(trace => sprintf q(%s %i|%i|%s),
